//
// VTrace.h
//
// Contains all the information about VTrace logs that could be used
// from user level (as opposed to kernel level).
//

///////////////////////////////////////////////////////////////////////////
//                            LOG ENTRY SIZE
///////////////////////////////////////////////////////////////////////////

#define MAX_LOG_ENTRY_SIZE        260
#define MAX_EVENT_LENGTH          (MAX_LOG_ENTRY_SIZE - 4)

///////////////////////////////////////////////////////////////////////////
//                            DEVICE CONTROL
///////////////////////////////////////////////////////////////////////////

// Device type values.  Note that values used by Microsoft Corporation
// are in the range 0-32767, and 32768-65535 are reserved for use
// by customers.

#define FILE_DEVICE_VTRACE_LOG    0x0000A472
#define FILE_DEVICE_VTRACE_RAWD   0x0000A473
#define FILE_DEVICE_VTRACE_INIT   0x0000A474

// Device control codes that can be sent to VTrcLog

#define VTRACE_CTRL_REMOVE_REASONS_TO_STOP \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x00, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_ADD_REASONS_TO_STOP \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x01, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_LOG_EVENT \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x02, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_READ_LOG \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x03, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_GET_SHARED_STATE \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x04, METHOD_NEITHER, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_FLUSH \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x05, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_GET_ACTIVITY \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x06, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_SET_USER_DEFINED_MASK \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x07, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_GET_MASK \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x08, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_INDICATE_USER_ACTIVITY \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x09, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_GET_IDLE_TIME \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x0A, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_RELOAD_REGISTRY \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x0B, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_TIME_LOG_EVENT \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x0C, METHOD_BUFFERED, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_GET_KERNEL_USER_SHARED_STATE \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x0D, METHOD_NEITHER, \
                         FILE_ANY_ACCESS)
#define VTRACE_CTRL_CHANGE_TO_EVENT_MASK_NUMBER \
        (ULONG) CTL_CODE(FILE_DEVICE_VTRACE_LOG, 0x0F, METHOD_NEITHER, \
                         FILE_ANY_ACCESS)

///////////////////////////////////////////////////////////////////////////
//                            LOG ENTRY TYPES
///////////////////////////////////////////////////////////////////////////

// Log entry types serve as the first byte (after the time) in a log entry,
// to identify what kind of entry it is

#define ENTRY_TYPE_MASKED_WHEN_USER_IDLE                5

#define ENTRY_TYPE_PROCESS_CREATE                       10
#define ENTRY_TYPE_PROCESS_DESTROY                      11
#define ENTRY_TYPE_THREAD_CREATE                        12
#define ENTRY_TYPE_THREAD_DESTROY                       13
#define ENTRY_TYPE_THREAD_SWITCH                        14
#define ENTRY_TYPE_PROCESS_SET_PRIORITY                 15
#define ENTRY_TYPE_THREAD_SET_PRIORITY                  16

#define ENTRY_TYPE_MESSAGE_GET_NOFILTER_CALL            30
#define ENTRY_TYPE_MESSAGE_GET_FILTER_CALL              31
#define ENTRY_TYPE_MESSAGE_GET_RETURN                   32
#define ENTRY_TYPE_MESSAGE_PEEK_REMOVE_NOFILTER_CALL    33
#define ENTRY_TYPE_MESSAGE_PEEK_REMOVE_FILTER_CALL      34
#define ENTRY_TYPE_MESSAGE_PEEK_NOREMOVE_NOFILTER_CALL  35
#define ENTRY_TYPE_MESSAGE_PEEK_NOREMOVE_FILTER_CALL    36
#define ENTRY_TYPE_MESSAGE_PEEK_RETURN                  37
#define ENTRY_TYPE_MESSAGE_DISPATCH_CALL                38
#define ENTRY_TYPE_MESSAGE_DISPATCH_RETURN              39
#define ENTRY_TYPE_MESSAGE_TRANSLATE_ACCEL_CALL         40
#define ENTRY_TYPE_MESSAGE_TRANSLATE_ACCEL_RETURN       41
#define ENTRY_TYPE_MESSAGE_TRANSLATE_CALL               42
#define ENTRY_TYPE_MESSAGE_TRANSLATE_RETURN             43
#define ENTRY_TYPE_MESSAGE_TRANSLATE_MDI_CALL           44
#define ENTRY_TYPE_MESSAGE_TRANSLATE_MDI_RETURN         45
#define ENTRY_TYPE_MESSAGE_SET_TIMER                    46
#define ENTRY_TYPE_MESSAGE_CANCEL_TIMER                 47
#define ENTRY_TYPE_MESSAGE_WAIT_CALL                    48
#define ENTRY_TYPE_MESSAGE_WAIT_RETURN                  49
#define ENTRY_TYPE_MESSAGE_GET_INPUT_STATE              50
#define ENTRY_TYPE_MESSAGE_GET_QUEUE_STATUS             51

#define ENTRY_TYPE_KEY_PRESS                            60

#define ENTRY_TYPE_CURSOR_LOAD                          70
#define ENTRY_TYPE_CURSOR_SET                           71

#define ENTRY_TYPE_WINDOW_CREATE                        80
#define ENTRY_TYPE_DIALOG_CREATE                        81

#define ENTRY_TYPE_FILE_COMPLETE_OPERATION              90
#define ENTRY_TYPE_FILE_OPEN                            91
#define ENTRY_TYPE_FILE_READ                            92
#define ENTRY_TYPE_FILE_WRITE                           93
#define ENTRY_TYPE_FILE_CLOSE                           94
#define ENTRY_TYPE_FILE_QUERY_INFO                      95
#define ENTRY_TYPE_FILE_SET_INFO                        96
#define ENTRY_TYPE_FILE_DIRECTORY_CONTROL               97
#define ENTRY_TYPE_FILE_NAME                            98
#define ENTRY_TYPE_FILE_INFORMATION                     99
#define ENTRY_TYPE_FILE_OPEN_QUERY_CLOSE                100
#define ENTRY_TYPE_FILE_RENAME                          101
#define ENTRY_TYPE_FILE_DELETE                          102
#define ENTRY_TYPE_FILE_FLUSH                           103
#define ENTRY_TYPE_FILE_LOCK                            104
#define ENTRY_TYPE_FILE_UNLOCK                          105
#define ENTRY_TYPE_FILE_SET_POSITION                    106
#define ENTRY_TYPE_FILE_LINK                            107
#define ENTRY_TYPE_FILE_COMPLETE_MDL_OPERATION          108
#define ENTRY_TYPE_FILE_FCB_INFORMATION                 109

#define ENTRY_TYPE_MESSAGE_POST_WINDOW                  110
#define ENTRY_TYPE_MESSAGE_POST_THREAD                  111
#define ENTRY_TYPE_MESSAGE_SEND                         112

#define ENTRY_TYPE_NETOBJ_COMPLETE_OPERATION            120
#define ENTRY_TYPE_NETOBJ_OPEN                          121
#define ENTRY_TYPE_NETOBJ_CLOSE                         122
#define ENTRY_TYPE_NETOBJ_CONNECT                       123
#define ENTRY_TYPE_NETOBJ_DISCONNECT                    124
#define ENTRY_TYPE_NETOBJ_SEND                          125
#define ENTRY_TYPE_NETOBJ_SEND_DATAGRAM                 126
#define ENTRY_TYPE_NETOBJ_RECEIVE                       127
#define ENTRY_TYPE_NETOBJ_RECEIVE_DATAGRAM              128
#define ENTRY_TYPE_NETOBJ_LISTEN                        129
#define ENTRY_TYPE_NETOBJ_ACCEPT                        130
#define ENTRY_TYPE_NETOBJ_ASSOCIATE_ADDRESS             131
#define ENTRY_TYPE_NETOBJ_DISASSOCIATE_ADDRESS          132

#define ENTRY_TYPE_NETOBJ_NOTIFY_CONNECT                140
#define ENTRY_TYPE_NETOBJ_NOTIFY_DISCONNECT             141
#define ENTRY_TYPE_NETOBJ_NOTIFY_RECEIVE                142
#define ENTRY_TYPE_NETOBJ_NOTIFY_RECEIVE_DATAGRAM       143
#define ENTRY_TYPE_NETOBJ_NOTIFY_RECEIVE_EXPEDITED      144

#define ENTRY_TYPE_VTRACE_ACTION                        160
#define ENTRY_TYPE_DEVICE_HOOK                          161
#define ENTRY_TYPE_VTRACE_VERSION                       162
#define ENTRY_TYPE_LOCAL_TIME                           163
#define ENTRY_TYPE_VTRACE_SET_MASK                      164
#define ENTRY_TYPE_BEAT                                 165
#define ENTRY_TYPE_DEVICE_UNHOOK                        166
#define ENTRY_TYPE_HIGH_TIMESTAMP                       167

#define ENTRY_TYPE_SYSTEM_CALL                          170
#define ENTRY_TYPE_SET_CPU_SPEED                        171
#define ENTRY_TYPE_USER_CHANGED                         172

#define ENTRY_TYPE_WAITOBJ_SIGNAL                       180
#define ENTRY_TYPE_WAITOBJ_SET_TIMER                    181
#define ENTRY_TYPE_WAITOBJ_CANCEL_TIMER                 182
#define ENTRY_TYPE_WAITOBJ_WAIT_CALL                    183
#define ENTRY_TYPE_WAITOBJ_WAIT_RETURN                  184
#define ENTRY_TYPE_WAITOBJ_WAIT_GENERIC_CALL            185
#define ENTRY_TYPE_WAITOBJ_WAIT_GENERIC_RETURN          186
#define ENTRY_TYPE_MSG_AND_WAITOBJ_WAIT_CALL            187
#define ENTRY_TYPE_MSG_AND_WAITOBJ_WAIT_RETURN          188

#define ENTRY_TYPE_RAWDISK_READ                         200
#define ENTRY_TYPE_RAWDISK_WRITE                        201
#define ENTRY_TYPE_RAWDISK_COMPLETE_OPERATION           202

#define ENTRY_TYPE_SECTION_CREATE                       210
#define ENTRY_TYPE_SECTION_OPEN                         211
#define ENTRY_TYPE_SECTION_MAP_VIEW                     212
#define ENTRY_TYPE_SECTION_UNMAP_VIEW                   213
#define ENTRY_TYPE_SECTION_GET_MDL                      214

#define ENTRY_TYPE_TESTING                              250

///////////////////////////////////////////////////////////////////////////
//                      FIELD MEANINGS IN ENTRY DATA
///////////////////////////////////////////////////////////////////////////

// These values are used in the create-network-object entry
// to identify what kind of object is being created

#define TRANSPORT_ADDRESS_OBJ_TYPE                      1
#define CONNECTION_ENDPOINT_OBJ_TYPE                    2
#define CONTROL_CHANNEL_OBJ_TYPE                        3
#define UNKNOWN_OBJ_TYPE                                4

// Flags used in I/O-completed log entries

#define FLAG_COMPLETED_SUCCESSFULLY                     1
#define FLAG_FAST_IO                                    8
#define FLAG_CANCELLED                                  16

// Types of VTrace actions

#define ACTION_STARTED_LOGGING                          1
#define ACTION_STOPPED_LOGGING                          2
#define ACTION_STARTED_FLUSHING                         3
#define ACTION_STOPPED_FLUSHING                         4

// Reasons for stopping logging

#define BECAUSE_BUFFER_OVERFLOWED                       1
#define BECAUSE_USER_REQUESTED                          2
#define BECAUSE_USER_IDLE                               4
#define BECAUSE_DISK_FULL                               8
#define BECAUSE_FLUSH_FAILED                            16
#define BECAUSE_SYSTEM_HALTING                          32
#define BECAUSE_VTRCLOG_UNLOADED                        64
#define BECAUSE_ACCESS_REQUIRED                         128
#define BECAUSE_END_NOT_FLUSHED                         256
#define BECAUSE_TRACE_NOT_INITIALIZED                   512
#define BECAUSE_MASK_CHANGED                            1024
#define BECAUSE_UNINSTALLING                            2048
#define BECAUSE_HIBERNATING                             4096

#define REASONS_TO_END_TRACE        (BECAUSE_USER_REQUESTED | \
                                     BECAUSE_ACCESS_REQUIRED | \
                                     BECAUSE_SYSTEM_HALTING | \
                                     BECAUSE_UNINSTALLING | \
                                     BECAUSE_HIBERNATING )

// Flags indicating what modifiers are on

#define SHIFT_MODIFIER_ON                               1
#define CTRL_MODIFIER_ON                                2
#define ALT_MODIFIER_ON                                 4

// Codes to distinguish system calls (they indicate the system call number
// in NT SP3)

#define FLUSH_ICACHE_SYSTEM_CALL_CODE                   0x36
#define FLUSH_WRITE_BUFFER_SYSTEM_CALL_CODE             0x39
#define TERMINATE_PROCESS_SYSTEM_CALL_CODE              0xBB
#define TERMINATE_THREAD_SYSTEM_CALL_CODE               0xBC
#define WRITE_REQUEST_DATA_SYSTEM_CALL_CODE             0xCA
#define WRITE_VM_SYSTEM_CALL_CODE                       0xCB

// Codes distinguishing file position types

#define POSITION_END_OF_FILE                            1
#define POSITION_ALLOCATION_SIZE                        2
#define POSITION_FILE_POINTER                           3

// Special file information bytes

#define FILE_INFORMATION_SECURITY_CODE                  100

// Codes for compressed access mask

#define COMPRESSED_ACCESS_MASK_READ                     '\1'
#define COMPRESSED_ACCESS_MASK_WRITE                    '\2'
#define COMPRESSED_ACCESS_MASK_DELETE                   '\4'

// The following macros indicate the meaning of the bits in the entry
// type info elements of the global structure. The "mask" ones indicate,
// if set, that the corresponding event type should be masked out (not
// logged) while that mask is active.

#define EVENT_MASK_BIT_FOR_MASK_0                       1
#define EVENT_MASK_BIT_FOR_MASK_1                       2
#define EVENT_MASK_BIT_FOR_USER_DEFINED_MASK            16
#define EVENT_MASK_BIT_FOR_USER_IDLE_MASK               32
#define EVENT_MASK_BIT_FOR_RECORD_STATE_MASK            64

// Indicator that disk or partition number is unknown

#define UNKNOWN_DEVICE_NUMBER                           255

/////////////////////////////////////////////////////////////////////////
//                     OTHER IMPORTANT LOGGING CONSTANTS
/////////////////////////////////////////////////////////////////////////

// The maximum number of waitable object id's recorded in a trace entry:

#define MAX_WAITABLE_OBJECTS_RECORDED                   25

// The maximum length of a process to log:

#define MAX_PROCESS_NAME_LENGTH                         128

// The maximum length of a user name to log:

#define MAX_USER_NAME_LENGTH                            128

// The number of milliseconds to wait for a stop-logging command to work:

#define PAUSE_FOR_FLUSHING_MS                           1500

///////////////////////////////////////////////////////////////////////////
//                            STRUCTURES
///////////////////////////////////////////////////////////////////////////

#define MAX_ACTIVITY_THREADS                            1024

typedef struct _kernel_user_shared_state {
  unsigned long maxThreads;
  volatile unsigned char msgChecksSinceActivity[MAX_ACTIVITY_THREADS];
} KERNEL_USER_SHARED_STATE, *PKERNEL_USER_SHARED_STATE;
