//
// VTrcDrv.h
//
// Contains structures that need to be shared between VTrace drivers.
//

/////////////////////////////////////////////////////////////////////////
//                                STRUCTURES
/////////////////////////////////////////////////////////////////////////

// The SHARED_STATE structure is obtained from VTrcLog by the other
// drivers so that they can share this state.

typedef PCHAR (*LOG_FUNC_TYPE) (char, ULONG);
typedef void (*SIMPLE_FUNC_TYPE) (void);
typedef BOOLEAN (*CHECK_IF_LOGGING_FUNC_TYPE) (char);
typedef void (*HIBERNATION_NOTIFY_TYPE) (BOOLEAN);

typedef struct _shared_state {
  KSPIN_LOCK mainMutex;
    // controls access to the following variables:  globals.logEntryBuffer[],
    //   globals.whyLoggingDisabled, globals.flushInProgress
    // NOTE: THIS MUST BE THE FIRST ITEM IN THIS STRUCTURE, OR THE
    // OPTIMIZATION OF USING &sharedState INSTEAD of &sharedState->mainMutex
    // IN THE NEW CONTEXT SWAP ROUTINE WON'T WORK!!!

  volatile LOG_FUNC_TYPE logEventFunc;        // to put entries in the log
  volatile CHECK_IF_LOGGING_FUNC_TYPE checkIfLoggingFunc;
                                              // to determine if logging of
                                              // this event type is on
  volatile SIMPLE_FUNC_TYPE emptyFSHashFunc;  // to clear the VTrcFS hash table
  volatile SIMPLE_FUNC_TYPE userActivityFunc; // to indicate user activity
  volatile HIBERNATION_NOTIFY_TYPE hibernationNotifyFunc;
                                              // to notify if hibernating
  volatile PDRIVER_OBJECT fsDriverObject;     // the VTrcFS driver
  volatile PDRIVER_OBJECT netDriverObject;    // the VTrcNet driver
  volatile ULONG filteredCreates;             // number of creates filtered
  KERNEL_USER_SHARED_STATE kernelUserSharedState;
} SHARED_STATE;

// The structure FS_HOOK_EXTENSION forms the device extension of any
// file system device associated with the VTrcFS driver. It contains the
// device attached to, the disk number, and the partition number.

typedef struct _fs_hook_extension {
  PDEVICE_OBJECT attachedDevice;
  unsigned long diskNumber;
  unsigned long partitionNumber;
  KEVENT PagingPathCountEvent;
  ULONG  PagingPathCount;
} FS_HOOK_EXTENSION, *PFS_HOOK_EXTENSION;         

// The VTRCNET_FILE_OBJECT_INFO structure contains information about
// a network file object.  Most importantly, it contains the callback
// routines that have been overridden by hook routines in VTrcNet.

typedef struct _vtrcnet_file_object_info {
  PFILE_OBJECT fileObject;
  struct _vtrcnet_file_object_info *nextInfo;
  PVOID connectEventHandler;
  PVOID connectEventContext;
  PVOID disconnectEventHandler;
  PVOID disconnectEventContext;
  PVOID receiveEventHandler;
  PVOID receiveEventContext;
  PVOID receiveDatagramEventHandler;
  PVOID receiveDatagramEventContext;
  PVOID receiveExpeditedEventHandler;
  PVOID receiveExpeditedEventContext;
  PVOID chainedReceiveEventHandler;
  PVOID chainedReceiveEventContext;
  PVOID chainedReceiveDatagramEventHandler;
  PVOID chainedReceiveDatagramEventContext;
  PVOID chainedReceiveExpeditedEventHandler;
  PVOID chainedReceiveExpeditedEventContext;
} VTRCNET_FILE_OBJECT_INFO;

// The structure NET_HOOK_EXTENSION forms the device extension of
// any network device associated with the VTrcNet driver.  It contains
// the device attached to and the head of a linked list of file object
// information structures.

typedef struct _net_hook_extension {
  PDEVICE_OBJECT attachedDevice;
  VTRCNET_FILE_OBJECT_INFO *fileObjectInfoList;
} NET_HOOK_EXTENSION, *PNET_HOOK_EXTENSION;         

/////////////////////////////////////////////////////////////////////////
//                              MACROS
/////////////////////////////////////////////////////////////////////////

#define LOG_EVENT           (*(sharedState->logEventFunc))
