//
// ConfigDlg.cpp : implements the CConfigDlg class.
//

#include "stdafx.h"
#include <winioctl.h>
#include <winsvc.h>
#include "Config.h"
#include "ConfigDlg.h"
#include "Helpers.h"
#include "../Common/RegConstraints.h"
#include "../Common/VTrcSrvc.h"
#include "../Common/VTrace.h"

#define NET_API_STATUS   DWORD
#define NET_API_FUNCTION __stdcall

#include <lmerr.h>
#include <lmwksta.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//                       BASIC DIALOG FUNCTIONS
/////////////////////////////////////////////////////////////////////////////

CConfigDlg::CConfigDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CConfigDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CConfigDlg)
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  m_hKey = NULL;
}

CConfigDlg::~CConfigDlg()
{
  if (m_hKey != NULL)
    RegCloseKey(m_hKey);
}

void CConfigDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CConfigDlg)
  DDX_Check(pDX, IDC_CHECK_AUTOMATIC_UPLOAD, m_automaticUpload);
  DDX_Text(pDX, IDC_EDIT_PROXY_SERVER, m_proxyServer);
  DDX_Text(pDX, IDC_EDIT_PROXY_PORT, m_proxyPort);
  DDX_Text(pDX, IDC_EDIT_TRANSFER_SIZE_LIMIT, m_transferSizeLimit);
  DDX_Text(pDX, IDC_EDIT_FREE_SPACE, m_freeSpace);
  DDX_Text(pDX, IDC_EDIT_LOG_BUFFER_SIZE, m_logBufferSize);
  DDX_Text(pDX, IDC_EDIT_TRACE_PIECE_SIZE_LIMIT, m_tracePieceSizeLimit);
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CConfigDlg, CDialog)
  //{{AFX_MSG_MAP(CConfigDlg)
  ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_STATIC_AUTOMATIC_UPLOAD, OnHelpAutomaticUpload)
  ON_BN_CLICKED(IDC_STATIC_PROXY_SERVER, OnHelpProxy)
  ON_BN_CLICKED(IDC_STATIC_TRANSFER_SIZE_LIMIT, OnHelpTransferSizeLimit)
  ON_BN_CLICKED(IDC_STATIC_FREE_SPACE, OnHelpFreeSpace)
  ON_BN_CLICKED(IDC_STATIC_LOG_BUFFER_SIZE, OnHelpLogBufferSize)
  ON_BN_CLICKED(IDC_BUTTON_HELP, OnButtonHelp)
  ON_BN_CLICKED(IDC_BUTTON_APPLY, OnButtonApply)
  ON_BN_CLICKED(IDC_BUTTON_RESET, OnButtonReset)
  ON_BN_CLICKED(IDC_STATIC_TRACE_PIECE_SIZE_LIMIT, OnHelpTracePieceSizeLimit)
  ON_BN_CLICKED(IDC_STATIC_PROXY_PORT, OnHelpProxy)
  ON_WM_PAINT()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//                            MESSAGE HANDLERS
/////////////////////////////////////////////////////////////////////////////

BOOL CConfigDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon

  LONG status;

  // Open the VTrace registry key for use by this program

  status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                        "Software\\Jay Lorch\\VTrace\\CurrentVersion", 0,
                        KEY_QUERY_VALUE | KEY_SET_VALUE, &m_hKey);
  if (status != ERROR_SUCCESS) {
    SetLastError(status);
    DisplayError("Cannot open VTrace registry key");
    m_hKey = NULL;
    EndDialog(-1);
    return TRUE;
  }

  // Read the parameters from the registry

  ReadFromRegistry();

  // Update the display to reflect the values in the registry

  UpdateData(FALSE);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.

HCURSOR CConfigDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

// CConfigDlg::OnHelpAutomaticUpload is called when the automatic upload
// field name is clicked.  It displays a help message describing what the
// significance of this parameter is.

void CConfigDlg::OnHelpAutomaticUpload() 
{
  MessageBox(
    "This checkbox indicates whether trace files should be automatically\n"
    "uploaded via the Web during computer idle time.  You may want to turn\n"
    "this feature off if, for instance, your machine is not always\n"
    "connected to the Internet.\n"
    "\n"
    "If you turn automatic uploading off by clearing the checkbox, then you\n"
    "will have to periodically upload trace files manually to remove them\n"
    "from your hard disk.  This involves running the upload utility in your\n"
    "Start menu, in the VTrace folder.",
    "Help: Automatic Upload");
}

// CConfigDlg::OnHelpProxy is called when the proxy server or proxy port
// field name is clicked.  It displays a help message describing what the
// significance of these parameters are.

void CConfigDlg::OnHelpProxy() 
{
  MessageBox(
    "VTrace uploads its trace files via the World Wide Web.  The Web can be\n"
    "accessed directly, or it can be accessed indirectly via a server\n"
    "called a \"proxy\".  Some machines, especially ones behind firewalls,\n"
    "must use a proxy in order to communicate with the Web.\n"
    "\n"
    "Usually, you will not want VTrace to use a proxy, in which case you\n"
    "should simply leave the proxy server field blank.  If you do want\n"
    "VTrace to use a proxy, e.g. if you are behind a firewall, you will\n"
    "have to enter the machine name and port number of that proxy in the\n"
    "fields here.\n"
    "\n"
    "To figure out the name and port number of your proxy, you can look and\n"
    "see what proxy your browser is using.  For example, on Netscape, you\n"
    "can select Edit->Preferences, then Advanced->Proxies, then check out\n"
    "what proxy name and port number it is using for HTTP access.",
    "Help: Proxy Server / Proxy Port Number");
}

// CConfigDlg::OnHelpTransferSizeLimit is called when the maximum transfer
// size field name is clicked.  It displays a help message describing what
// the significance of this parameter is.

void CConfigDlg::OnHelpTransferSizeLimit() 
{
  MessageBox(
    "When VTrace uploads trace files via the Web, it does so by opening\n"
    "a connection to the upload server and transferring a chunk of a trace\n"
    "file.  The more it can transfer in one chunk, the less overhead it\n"
    "incurs for opening multiple connections, but the more likely it is to\n"
    "fail in the middle of a connection and have to retry the entire\n"
    "connection.\n"
    "\n"
    "We recommend reducing this value from its default if either of the\n"
    "following conditions exists:\n"
    "  1)  Your connection to the Web is flaky and prone to disconnection.\n"
    "  2)  You connect to the Web via a proxy that limits the amount of\n"
    "       data you can send in one open connection.\n",
    "Help: Transfer Size Limit");
}

// CConfigDlg::OnHelpTracePieceSizeLimit is called when the maximum trace
// piece size field name is clicked.  It displays a help message describing
// what the significance of this parameter is.

void CConfigDlg::OnHelpTracePieceSizeLimit() 
{
  MessageBox(
    "When VTrace's traces are getting too long, it divides them into pieces\n"
    "with suffixes .001, .002, etc. There are two reasons for having small\n"
    "pieces like this. First, while a file is being compressed, the file\n"
    "exists alongside its compressed version, which is usually about half\n"
    "its size. So, if a file were 500 MB, while compressing there would be\n"
    "an additional 250 MB or so taken up by the compressed version. The\n"
    "second reason is that until a whole file has been uploaded via the\n"
    "internet, it must remain on your hard drive. The shorter the file, the\n"
    "sooner it can be uploaded.\n",
    "Help: Trace File Size Limit");
}

// CConfigDlg::OnHelpFreeSpace is called when the free space field name is
// clicked.  It displays a help message describing what the free space field
// does.

void CConfigDlg::OnHelpFreeSpace() 
{
  MessageBox(
    "VTrace is continuously generating megabyte upon megabyte of trace\n"
    "files.  In so doing, there is the danger it will fill up the drive\n"
    "it's using for those trace files, making it hard for you to do your\n"
    "work.  For this reason, you can set a minimum amount of free space\n"
    "that VTrace makes sure to leave available for you.  In other words, if\n"
    "the free space on the drive goes below this amount, VTrace will stop\n"
    "tracing until more space becomes available.",
    "Help: Free Disk Space");
}

// CConfigDlg::OnHelpLogBufferSize is called when the log buffer size field
// name is clicked.  It displays a help message describing what the log
// buffer size field does.

void CConfigDlg::OnHelpLogBufferSize() 
{
  MessageBox(
    "If VTrace wrote its traces directly to your disk all the time, the\n"
    "disk would be continually busy.  For this reason, VTrace keeps a\n"
    "buffer of recent log events in memory, and only periodically writes\n"
    "the contents of that buffer to the disk (when the buffer is getting\n"
    "really full).  A large log buffer means VTrace has to access your\n"
    "disk less often, but it also means less memory is available for your\n"
    "other applications to use.  Depending on how much memory you have,\n"
    "you may want to vary the size of this log buffer from its default.\n"
    "\n"
    "Note that changes to this parameter do not take effect until you\n"
    "reboot the computer.",
    "Help: Log Buffer Size");
}

// OnButtonReset resets all the fields in the display to the defaults.

void CConfigDlg::OnButtonReset() 
{
  m_automaticUpload = TRUE;
  m_proxyServer = "";
  m_proxyPort = 0;
  m_transferSizeLimit = DEFAULT_TRANSFER_SIZE_LIMIT;
  m_freeSpace = DEFAULT_FREE_SPACE;
  m_logBufferSize = DEFAULT_LOG_BUFFER_SIZE;
  m_tracePieceSizeLimit = DEFAULT_TRACE_PIECE_SIZE_LIMIT;
  UpdateData(FALSE);
}

// OnButtonApply is called when the user clicks the Apply button.  It checks
// to make sure the fields entered are valid, and, if so, applies them to
// the registry.

void CConfigDlg::OnButtonApply()
{
  UpdateData(TRUE);
  if (CheckFields())
    ApplyChanges();
  else
    MessageBox("Changes not applied.", "Information", MB_OK);
}

// OnOK is called when the user clicks OK.  It applies the changes and exits
// the dialog.  If the changes are not valid, however, it displays an error
// message and does not exit.

void CConfigDlg::OnOK() 
{
  UpdateData(TRUE);
  if (CheckFields()) {
    ApplyChanges();
    CDialog::OnOK();
  }
  else
    MessageBox("Changes not applied.", "Information", MB_OK);
}

// OnButtonHelp is called when the user clicks the Help button.  It displays
// a general help message, which, among other things, describes how to get
// more detailed help.

void CConfigDlg::OnButtonHelp() 
{
  MessageBox(
    "This program allows you to change the internal configuration of\n"
    "VTrace.  To do so, enter your changes in the fields presented, then\n"
    "click OK or Apply to enter them to VTrace.\n"
    "\n"
    "If you want to restore all fields to their defaults, click the Reset\n"
    "to Defaults button.  You may then click OK or Apply to apply them to\n"
    "VTrace.\n"
    "\n"
    "To get help about a particular field, click the field name.\n",
    "Help: General");
}

/////////////////////////////////////////////////////////////////////////////
//                               HELPERS
/////////////////////////////////////////////////////////////////////////////

// CConfigDlg::ReadFromRegistry reads the parameters from the registry into
// the object fields m_whatever.

void CConfigDlg::ReadFromRegistry (void)
{
  DWORD type, length, n;
  LONG status;
  char serverName[MAX_SERVER_LENGTH];

  // Read the automatic upload parameter (called AlwaysConnected for legacy
  // reasons), defaulting to TRUE if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "AlwaysConnected", 0, &type, (BYTE *) &n,
                           &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_automaticUpload = TRUE;
  else
    m_automaticUpload = (n != 0);

  // Read the proxy name, defaulting to "" if not found

  length = MAX_SERVER_LENGTH;
  status = RegQueryValueEx(m_hKey, "ProxyServer", 0, &type,
                           (BYTE *) serverName, &length);
  if (status == ERROR_SUCCESS && type == REG_SZ)
    m_proxyServer = serverName;
  else
    m_proxyServer = "";

  // Read the proxy port, defaulting to DEFAULT_PROXY_PORT if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "ProxyPort", 0, &type,
                           (BYTE *) &m_proxyPort, &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_proxyPort = DEFAULT_PROXY_PORT;

  // Read the transfer size limit, defauting if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "MaxTransferKB", 0, &type,
                           (BYTE *) &m_transferSizeLimit, &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_transferSizeLimit = DEFAULT_TRANSFER_SIZE_LIMIT;

  // Read the free space needed, defaulting if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "FreeMBNeeded", 0, &type,
                           (BYTE *) &m_freeSpace, &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_freeSpace = DEFAULT_FREE_SPACE;

  // Read the log buffer size, defaulting if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "LogBufferKB", 0, &type,
                           (BYTE *) &m_logBufferSize, &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_logBufferSize = DEFAULT_LOG_BUFFER_SIZE;

  // Read the trace piece size limit, defaulting if not found

  length = 4;
  status = RegQueryValueEx(m_hKey, "MaxTracePieceMB", 0, &type,
                           (BYTE *) &m_tracePieceSizeLimit, &length);
  if (status != ERROR_SUCCESS || type != REG_DWORD || length != 4)
    m_tracePieceSizeLimit = DEFAULT_TRACE_PIECE_SIZE_LIMIT;
}

// CheckFields checks to make sure all the fields entered by the user are
// valid.  It returns a boolean indicating whether they are.

BOOL CConfigDlg::CheckFields()
{
  CString errString;
  BOOL retVal = TRUE;

  // Check that the proxy server name is not too long

  if (m_proxyServer.GetLength() >= MAX_SERVER_LENGTH) {
    errString.Format("Proxy server name must have fewer than %d characters.\n"
                     "If this is a problem, try using the IP address.",
                     MAX_SERVER_LENGTH);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }

  // Check that the transfer size limit is within the correct range

  if (m_transferSizeLimit < MIN_TRANSFER_SIZE_LIMIT) {
    errString.Format("Transfer size limit must be at least %d KB.",
                     MIN_TRANSFER_SIZE_LIMIT);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }
  else if (m_transferSizeLimit > MAX_TRANSFER_SIZE_LIMIT) {
    errString.Format("Transfer size limit must be at most %d KB.",
                     MAX_TRANSFER_SIZE_LIMIT);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }

  // Check that the free space is within the correct range

  if (m_freeSpace < MIN_FREE_SPACE) {
    errString.Format("Free space maintained on disk must be at least %d MB.",
                     MIN_FREE_SPACE);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }
  else if (m_freeSpace > MAX_FREE_SPACE) {
    errString.Format("Free space maintained on disk must be at most %d MB.",
                     MAX_FREE_SPACE);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }

  // Check that the log buffer size is within the correct range

  if (m_logBufferSize < MIN_LOG_BUFFER_SIZE) {
    errString.Format("Log buffer size must be at least %d KB.",
                     MIN_LOG_BUFFER_SIZE);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }
  else if (m_logBufferSize > MAX_LOG_BUFFER_SIZE) {
    errString.Format("Log buffer size must be at most %d KB.",
                     MAX_LOG_BUFFER_SIZE);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }

  // Check that the trace piece size limit is within the correct range

  if (m_tracePieceSizeLimit < MIN_TRACE_PIECE_SIZE_LIMIT) {
    errString.Format("Trace file size limit must be at least %d MB.",
                     MIN_TRACE_PIECE_SIZE_LIMIT);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }
  else if (m_tracePieceSizeLimit > MAX_TRACE_PIECE_SIZE_LIMIT) {
    errString.Format("Trace file size limit must be at most %d MB.",
                     MAX_TRACE_PIECE_SIZE_LIMIT);
    MessageBox(errString, "Error", MB_OK | MB_ICONEXCLAMATION);
    retVal = FALSE;
  }

  return retVal;
}

// ApplyChanges applies the values in the m_whatever field to the registry.
// If the value is the default, it instead deletes the registry value.

void CConfigDlg::ApplyChanges()
{
  DWORD n, length;
  SC_HANDLE schService;
  SC_HANDLE schSCManager;
  SERVICE_STATUS serviceStatus;
  HANDLE hDevice;

  // Automatic upload value

  if (m_automaticUpload)
    RegDeleteValue(m_hKey, "AlwaysConnected");
  else {
    n = 0;
    RegSetValueEx(m_hKey, "AlwaysConnected", 0, REG_DWORD, (BYTE *) &n, 4);
  }

  // Proxy server

  length = m_proxyServer.GetLength();
  if (length == 0)
    RegDeleteValue(m_hKey, "ProxyServer");
  else
    RegSetValueEx(m_hKey, "ProxyServer", 0, REG_SZ,
                  (BYTE *) (LPCTSTR) m_proxyServer, length + 1);

  // Proxy port number

  if (m_proxyPort == DEFAULT_PROXY_PORT)
    RegDeleteValue(m_hKey, "ProxyPort");
  else
    RegSetValueEx(m_hKey, "ProxyPort", 0, REG_DWORD, (BYTE *) &m_proxyPort,
                  4);

  // Transfer size limit

  if (m_transferSizeLimit == DEFAULT_TRANSFER_SIZE_LIMIT)
    RegDeleteValue(m_hKey, "MaxTransferKB");
  else
    RegSetValueEx(m_hKey, "MaxTransferKB", 0, REG_DWORD,
                  (BYTE *) &m_transferSizeLimit, 4);

  // Free space

  if (m_freeSpace == DEFAULT_FREE_SPACE)
    RegDeleteValue(m_hKey, "FreeMBNeeded");
  else
    RegSetValueEx(m_hKey, "FreeMBNeeded", 0, REG_DWORD, (BYTE *) &m_freeSpace,
                  4);

  // Log buffer size

  if (m_logBufferSize == DEFAULT_LOG_BUFFER_SIZE)
    RegDeleteValue(m_hKey, "LogBufferKB");
  else
    RegSetValueEx(m_hKey, "LogBufferKB", 0, REG_DWORD,
                  (BYTE *) &m_logBufferSize, 4);

  // Trace piece size limit

  if (m_tracePieceSizeLimit == DEFAULT_TRACE_PIECE_SIZE_LIMIT)
    RegDeleteValue(m_hKey, "MaxTracePieceMB");
  else
    RegSetValueEx(m_hKey, "MaxTracePieceMB", 0, REG_DWORD,
                  (BYTE *) &m_tracePieceSizeLimit, 4);

  // Notify VTrcSrvc of the changes to the registry

  schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
  if (schSCManager != NULL) {
    schService = OpenService(schSCManager, "VTrcSrvc", SERVICE_ALL_ACCESS);
    if (schService != NULL) {
      ControlService(schService, SERVICE_CONTROL_RELOAD_REGISTRY,
                     &serviceStatus);
      CloseServiceHandle(schService);
    }
    CloseServiceHandle(schSCManager);
  }

  // Notify VTrcLog of the changes to the registry

  hDevice = CreateFile("\\\\.\\VTrcLog", GENERIC_READ | GENERIC_WRITE, 0, NULL,
                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hDevice != (HANDLE) -1) {
    DeviceIoControl(hDevice, VTRACE_CTRL_RELOAD_REGISTRY, NULL, 0, NULL,
                    0, &length, NULL);
    CloseHandle(hDevice);
  }
}
