//
// Globals.h contains macros and structure definitions that are of
// general use in the files for the VTrace DLL.
//

#include <imagehlp.h>

////////////////////////////////////////////////////////////////////////////
//                               MACROS
////////////////////////////////////////////////////////////////////////////

// The Xxx_INDEX macros define indices into the list of hook info
// structures.

#define GetMessageA_INDEX                           0
#define GetMessageW_INDEX                           1
#define PeekMessageA_INDEX                          2
#define PeekMessageW_INDEX                          3
#define MsgWaitForMultipleObjects_INDEX             4
#define MsgWaitForMultipleObjectsEx_INDEX           5
#define DispatchMessageA_INDEX                      6
#define DispatchMessageW_INDEX                      7
#define TranslateAccelerator_INDEX                  8
#define TranslateAcceleratorA_INDEX                 9
#define TranslateAcceleratorW_INDEX                10
#define TranslateMessage_INDEX                     11
#define TranslateMDISysAccel_INDEX                 12
#define WaitMessage_INDEX                          13
#define GetInputState_INDEX                        14
#define GetQueueStatus_INDEX                       15
#define LoadCursorA_INDEX                          16
#define LoadCursorW_INDEX                          17
#define LoadCursorFromFileA_INDEX                  18
#define LoadCursorFromFileW_INDEX                  19
#define SetCursor_INDEX                            20
#define CreateWindowExA_INDEX                      21
#define CreateWindowExW_INDEX                      22
#define CreateDialogParamA_INDEX                   23
#define CreateDialogParamW_INDEX                   24
#define CreateDialogIndirectParamA_INDEX           25
#define CreateDialogIndirectParamW_INDEX           26
#define CreateDialogIndirectParamAorW_INDEX        27
#define DialogBoxParamA_INDEX                      28
#define DialogBoxParamW_INDEX                      29
#define DialogBoxIndirectParamA_INDEX              30
#define DialogBoxIndirectParamW_INDEX              31
#define DialogBoxIndirectParamAorW_INDEX           32
#define SendMessageA_INDEX                         33
#define SendMessageW_INDEX                         34
#define SendMessageTimeoutA_INDEX                  35
#define SendMessageTimeoutW_INDEX                  36
#define SendMessageCallbackA_INDEX                 37
#define SendMessageCallbackW_INDEX                 38
#define PostMessageA_INDEX                         39
#define PostMessageW_INDEX                         40
#define PostThreadMessageA_INDEX                   41
#define PostThreadMessageW_INDEX                   42
#define SetTimer_INDEX                             43
#define KillTimer_INDEX                            44
#define LoadLibraryA_INDEX                         45
#define LoadLibraryW_INDEX                         46
#define LoadLibraryExA_INDEX                       47
#define LoadLibraryExW_INDEX                       48

#define NUMBER_OF_HOOKED_FUNCTIONS                 49
#define NUMBER_OF_USER32_HOOKED_FUNCTIONS          45

// The following is the number of hash buckets used in the hook hash
// functions. It's a prime number.

#define NUMBER_OF_HOOK_HASH_BUCKETS      163

////////////////////////////////////////////////////////////////////////////
//                                TYPES
////////////////////////////////////////////////////////////////////////////

// The following Xxx_PROC types are function pointer types for the different
// types of functions being hooked.

typedef BOOL (WINAPI *GetMessage_PROC)(LPMSG, HWND, UINT, UINT);
typedef BOOL (WINAPI *PeekMessage_PROC)(LPMSG, HWND, UINT, UINT, UINT);
typedef DWORD (WINAPI *MsgWaitForMultipleObjects_PROC)
        (DWORD, LPHANDLE, BOOL, DWORD, DWORD);
typedef DWORD (WINAPI *MsgWaitForMultipleObjectsEx_PROC)
        (DWORD, LPHANDLE, DWORD, DWORD, DWORD);
typedef LONG (WINAPI *DispatchMessage_PROC)(CONST MSG *);
typedef int (WINAPI *TranslateAccelerator_PROC)(HWND, HACCEL, LPMSG);
typedef BOOL (WINAPI *TranslateMessage_PROC)(CONST MSG *);
typedef BOOL (WINAPI *TranslateMDISysAccel_PROC)(HWND, LPMSG);
typedef BOOL (WINAPI *WaitMessage_PROC)(VOID);
typedef BOOL (WINAPI *GetInputState_PROC)(VOID);
typedef DWORD (WINAPI *GetQueueStatus_PROC)(UINT flags);
typedef HCURSOR (WINAPI *LoadCursorA_PROC)(HINSTANCE, LPCSTR);
typedef HCURSOR (WINAPI *LoadCursorW_PROC)(HINSTANCE, LPCWSTR);
typedef HCURSOR (WINAPI *LoadCursorFromFileA_PROC)(LPCSTR);
typedef HCURSOR (WINAPI *LoadCursorFromFileW_PROC)(LPCWSTR);
typedef HCURSOR (WINAPI *SetCursor_PROC)(HCURSOR);
typedef HWND (WINAPI *CreateWindowExA_PROC)(DWORD, LPCSTR, LPCSTR, DWORD, int,
                                            int, int ,int, HWND, HMENU,
                                            HINSTANCE, LPVOID);
typedef HWND (WINAPI *CreateWindowExW_PROC)(DWORD, LPCWSTR, LPCWSTR, DWORD,
                                            int, int, int ,int, HWND, HMENU,
                                            HINSTANCE, LPVOID);
typedef HWND (WINAPI *CreateDialogParamA_PROC)(HINSTANCE, LPCSTR, HWND,
                                               DLGPROC, LPARAM);
typedef HWND (WINAPI *CreateDialogParamW_PROC)(HINSTANCE, LPCWSTR, HWND,
                                               DLGPROC, LPARAM);
typedef HWND (WINAPI *CreateDialogIndirectParamAorW_PROC)
        (HINSTANCE, LPCDLGTEMPLATE, HWND, DLGPROC, LPARAM, DWORD);
typedef HWND (WINAPI *CreateDialogIndirectParamA_PROC)
        (HINSTANCE, LPCDLGTEMPLATEA, HWND, DLGPROC, LPARAM);
typedef HWND (WINAPI *CreateDialogIndirectParamW_PROC)
        (HINSTANCE, LPCDLGTEMPLATEW, HWND, DLGPROC, LPARAM);
typedef LRESULT (WINAPI *SendMessage_PROC)(HWND, UINT, WPARAM, LPARAM);
typedef LRESULT (WINAPI *SendMessageTimeout_PROC)(HWND, UINT, WPARAM, LPARAM,
                                                  UINT, UINT, LPDWORD);
typedef BOOL (WINAPI *SendMessageCallback_PROC)
        (HWND, UINT, WPARAM, LPARAM, SENDASYNCPROC, DWORD);
typedef BOOL (WINAPI *PostMessage_PROC)(HWND, UINT, WPARAM, LPARAM);
typedef BOOL (WINAPI *PostThreadMessage_PROC)(DWORD, UINT, WPARAM, LPARAM);
typedef UINT (WINAPI *SetTimer_PROC)(HWND, UINT, UINT, TIMERPROC);
typedef BOOL (WINAPI *KillTimer_PROC)(HWND, UINT);
typedef HMODULE (WINAPI *LoadLibraryA_PROC)(LPCSTR);
typedef HMODULE (WINAPI *LoadLibraryW_PROC)(LPCWSTR);
typedef HMODULE (WINAPI *LoadLibraryExA_PROC)(LPCSTR, HANDLE, DWORD);
typedef HMODULE (WINAPI *LoadLibraryExW_PROC)(LPCWSTR, HANDLE, DWORD);
typedef HANDLE (WINAPI *CreateFileA_PROC)
        (LPCSTR, DWORD, DWORD, LPSECURITY_ATTRIBUTES, DWORD, DWORD, HANDLE);
typedef HANDLE (WINAPI *CreateFileW_PROC)
        (LPCWSTR, DWORD, DWORD, LPSECURITY_ATTRIBUTES, DWORD, DWORD, HANDLE);
typedef HFILE (WINAPI *OpenFile_PROC)(LPCSTR, LPOFSTRUCT, UINT);
typedef BOOL (WINAPI *CloseHandle_PROC)(HANDLE);
typedef BOOL (WINAPI *BitBlt_PROC)
        (HDC, int, int, int, int, HDC, int, int, DWORD);
typedef int (WINAPI *DialogBoxParamA_PROC)(HINSTANCE, LPCSTR, HWND, DLGPROC,
                                           LPARAM);
typedef int (WINAPI *DialogBoxParamW_PROC)(HINSTANCE, LPCWSTR, HWND, DLGPROC,
                                           LPARAM);
typedef int (WINAPI *DialogBoxIndirectParamA_PROC)(HINSTANCE, LPCDLGTEMPLATEA,
                                                   HWND, DLGPROC, LPARAM);
typedef int (WINAPI *DialogBoxIndirectParamW_PROC)(HINSTANCE, LPCDLGTEMPLATEW,
                                                   HWND, DLGPROC, LPARAM);
typedef int (WINAPI *DialogBoxIndirectParamAorW_PROC)
        (HINSTANCE, LPCDLGTEMPLATE, HWND, DLGPROC, LPARAM, DWORD);
typedef BOOL (WINAPI *SetEvent_PROC)(HANDLE);
typedef BOOL (WINAPI *SetWaitableTimer_PROC)(HANDLE, const LARGE_INTEGER *,
                                             LONG, PTIMERAPCROUTINE, LPVOID,
                                             BOOL);
typedef BOOL (WINAPI *CancelWaitableTimer_PROC)(HANDLE);
typedef BOOL (WINAPI *ReleaseSemaphore_PROC)(HANDLE, LONG, LPLONG);
typedef BOOL (WINAPI *ReleaseMutex_PROC)(HANDLE);

////////////////////////////////////////////////////////////////////////////
//                         STRUCTURE DEFINITIONS
////////////////////////////////////////////////////////////////////////////

// The compaction per-process variables structure contains all the
// information each process needs to keep track of in order to allow
// the trace compaction via message queue check removal to work.

typedef struct _compaction_per_process_vars {
  BOOL cursorIsHourglass;         // is the cursor an hourglass?
  HCURSOR waitCursor;             // handle of the "wait" cursor   
  HCURSOR appStartingCursor;      // handle of the "app-starting" cursor
} COMPACTION_PER_PROCESS_VARS;

// The compaction per-thread variables structure contains all the
// information each thread needs to keep track of in order to allow
// the trace compaction via message queue check removal to work.

typedef struct _compaction_per_thread_vars {
  DWORD msgChecksSinceActivity;    // how many times has this thread checked
                                   //   the message queue in a row without
                                   //   doing anything?
} COMPACTION_PER_THREAD_VARS;

// The hooked-function info structure contains information about one
// hooked function.

typedef struct _hooked_function_info {
  PDWORD origFunctionAddress;          // address of original function
  PDWORD newFunctionAddress;           // address of hook function
  PCHAR name;                          // name of function
  struct _hooked_function_info *nextInOrigHash;
    // next structure in the same hash bucket, in the hash indexed by original
    // function address
  struct _hooked_function_info *nextInNewHash;
    // next structure in the same hash bucket, in the hash indexed by new
    // (hook) function address
} HOOKED_FUNCTION_INFO, *PHOOKED_FUNCTION_INFO;

////////////////////////////////////////////////////////////////////////////
//                      GLOBAL VARIABLE DECLARATIONS
////////////////////////////////////////////////////////////////////////////

// Whether we acquired all the resources we need to run properly:

extern BOOL initializedOK;

// The handle to the VTrcInit device:

extern HANDLE vtrcInitHandle;

// The per-process variables related to the compaction technique:

extern COMPACTION_PER_PROCESS_VARS compactionProcessVars;

// The index to thread-local storage to get the per-thread variables:

extern DWORD compactionThreadVarsPtrTlsIndex;

// A pointer to the kernel-user shared state in this address space:

extern PKERNEL_USER_SHARED_STATE kernelUserSharedState;

// The page size on the current machine:

extern DWORD pageSize;

// The following are the two hashes, each represented by an array of
// hash buckets, indexed on two different fields of the hooked-function
// info structures.  The "original" hash is indexed on the original
// function address; the "new" hash is indexed on the new (hook)
// function address.

extern HOOKED_FUNCTION_INFO *hookOrigHashBucket[NUMBER_OF_HOOK_HASH_BUCKETS];
extern HOOKED_FUNCTION_INFO *hookNewHashBucket[NUMBER_OF_HOOK_HASH_BUCKETS];

// The array of hooked-function info structures, one for each hook function:

extern HOOKED_FUNCTION_INFO hookInfo[NUMBER_OF_HOOKED_FUNCTIONS];
