//
// InstallDlg.cpp : implementats the CInstallDlg class
//

#include "stdafx.h"
#include "Install.h"
#include "InstallDlg.h"
#include "InstallRoutines.h"
#include "../Common/RegConstraints.h"
#include <winioctl.h>
#include <winsvc.h>
#include <shlobj.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//                           BASIC DIALOG FUNCTIONS
/////////////////////////////////////////////////////////////////////////////

CInstallDlg::CInstallDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CInstallDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CInstallDlg)
  m_mainDirectory = _T("");
  m_personalName = _T("");
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  installingThread = NULL;
}

void CInstallDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CInstallDlg)
  DDX_Text(pDX, IDC_EDIT1, m_mainDirectory);
  DDX_Text(pDX, IDC_EDIT2, m_personalName);
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CInstallDlg, CDialog)
  //{{AFX_MSG_MAP(CInstallDlg)
  ON_WM_QUERYDRAGICON()
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_BN_CLICKED(ID_BROWSE, OnBrowse)
  ON_BN_CLICKED(IDC_FREE_SPACE, OnFreeSpace)
  ON_MESSAGE(WM_CHANGE_STATUS, OnChangeStatus)
  ON_MESSAGE(WM_INSTALL_COMPLETE, OnInstallComplete)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//                             MESSAGE HANDLERS
/////////////////////////////////////////////////////////////////////////////

BOOL CInstallDlg::OnInitDialog()
{
  HKEY hKey;
  LONG status;
  DWORD type, length, n;
  char registryString[MAX_REGISTRY_STRING_LENGTH];

  // Initialize registry variables to reasonable values in case they
  // cannot be found in the registry

  m_mainDirectory = "C:\\VTrace";
  m_personalName = "";
  m_previousVersion = 0;
  m_formerMainDirectory = "";

  // Search the registry for information

  status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                        "Software\\Jay Lorch\\VTrace\\CurrentVersion",
                        0, KEY_QUERY_VALUE, &hKey);
  if (status == ERROR_SUCCESS) {
    length = MAX_REGISTRY_STRING_LENGTH;
    status = RegQueryValueEx(hKey, "MainDirectory", NULL, &type,
                             (BYTE *) registryString, &length);
    if (status == ERROR_SUCCESS && type == REG_SZ) {
      m_mainDirectory = registryString;
      m_formerMainDirectory = registryString;
    }

    length = MAX_REGISTRY_STRING_LENGTH;
    status = RegQueryValueEx(hKey, "PersonalName", NULL, &type,
                             (BYTE *) registryString, &length);
    if (status == ERROR_SUCCESS && type == REG_SZ)
      m_personalName = registryString;

    length = 4;
    status = RegQueryValueEx(hKey, "Version", NULL, &type, (BYTE *) &n,
                             &length);
    if (status == ERROR_SUCCESS && type == REG_DWORD && length == 4)
      m_previousVersion = n;

    RegCloseKey(hKey);
  }

  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon
  
  // TODO: Add extra initialization here
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.

HCURSOR CInstallDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

// CInstallDlg::OnOK is called when the OK button is clicked.  It launches a
// thread to install VTrace.

void CInstallDlg::OnOK() 
{
  CWnd *okButton;

  // Update the dialog object data to reflect the window

  UpdateData(TRUE);

  if (installingThread == NULL) {
    if (CheckFields()) {
      // Disable the OK button
      okButton = GetDlgItem(IDOK);
      if (okButton != NULL) okButton->EnableWindow(FALSE);

      // Launch the install thread and pass it the appropriate parameters
      installParameters.hWnd = m_hWnd;
      installParameters.mainDirectory = m_mainDirectory;
      installParameters.personalName = m_personalName;
      installParameters.previousVersion = m_previousVersion;
      installingThread = AfxBeginThread(&InstallAsynchronously,
                                        &installParameters);
    }
  }
}

// CInstallDlg::OnCancel is called when the Cancel button is clicked.  It
// terminates the installing thread (if it is running), then ends the program.

void CInstallDlg::OnCancel()
{
  if (installingThread != NULL) {
    TerminateThread(installingThread->m_hThread, -1);
    MessageBox("Installation cancelled.\n"
               "Run the Uninstall program to undo any partial installation.\n",
               "Information", MB_OK);
    installingThread = NULL;
  }
  
  CDialog::OnCancel();
}

// CInstallDlg::OnBrowse is called when the Browse button is clicked, to allow
// the user to browse for a new main VTrace directory.  If the user selects a
// directory that does not end in \VTrace, it appends \VTrace to it.  Finally,
// it updates the directory name in the dialog box and dialog variable.

void CInstallDlg::OnBrowse() 
{
  BROWSEINFO bi;
  LPITEMIDLIST finalIdList;
  char str[MAX_PATH];
  BOOL success;
  int length;

  bi.hwndOwner = m_hWnd;
  bi.pidlRoot = NULL;
  bi.pszDisplayName = str;
  bi.lpszTitle = "Select main VTrace directory";
  bi.ulFlags = BIF_RETURNONLYFSDIRS;
  bi.lpfn = NULL;
  bi.lParam = NULL;
  bi.iImage = NULL;

  finalIdList = SHBrowseForFolder(&bi);
  if (finalIdList == NULL) return;
  success = SHGetPathFromIDList(finalIdList, str);

  // The user has successfully browsed for a folder at this point,
  // and that folder name is in the string "str".

  if (success) {
    // If it does not already end in \VTrace, make it end so

    length = strlen(str);
    if (length < 7 || stricmp(&str[length-7], "\\VTrace")) {
      if (str[length-1] == '\\')
        strcat(str, "VTrace");
      else
        strcat(str, "\\VTrace");
    }
    SetDlgItemText(IDC_EDIT1, str);
    UpdateData(TRUE);
  }

  // SHFree(finalIdList);
}

// CInstallDlg::OnFreeSpace is called when the Check Free Space button is
// clicked.  It reports to the user the amount of free space in the chosen
// directory.

void CInstallDlg::OnFreeSpace()
{
  UINT freeMB;
  CString str;

  UpdateData(TRUE);

  // Check how much free space is in the directory

  if (GetFreeMB(m_mainDirectory, &freeMB)) {
    str.Format("%s has %d MB of free space.", m_mainDirectory, freeMB);
    MessageBox(str, "Free Space", MB_OK);
  }
  else
    DisplayError("Could not get free disk space");
}

// OnChangeStatus is called when a WM_CHANGE_STATUS message is posted to the
// dialog box.  The wParam field is a pointer to a string that the status
// message should be changed to.

void CInstallDlg::OnChangeStatus (WPARAM wParam, LPARAM lParam)
{
  SetDlgItemText(IDC_STATIC_PROGRESS, (char *) wParam);
}

// OnInstallComplete is called when a WM_INSTALL_COMPLETE message is posted to
// the dialog box.  It indicates that the installing thread is finished.  So,
// we clear the variable installingThread so we don't try to terminate it
// later.

void CInstallDlg::OnInstallComplete (WPARAM wParam, LPARAM lParam)
{
  installingThread = NULL;
  OnCancel();
}

/////////////////////////////////////////////////////////////////////////////
//                              FIELD CHECKING
/////////////////////////////////////////////////////////////////////////////

// CheckFields checks whether the fields supplied by the user are valid
// for doing an install.

BOOL CInstallDlg::CheckFields (void)
{
  char testFileName[MAX_PATH], errorString[128];
  UINT freeMB;
  int response;
  WIN32_FIND_DATA findData;
  HANDLE findHandle;

  // Make sure the user isn't trying to install to the same directory as this
  // installation program is running from

  sprintf(testFileName, "%s\\VTrcLog.sys", m_mainDirectory);
  findHandle = FindFirstFile(testFileName, &findData);
  if (findHandle != INVALID_HANDLE_VALUE) {
    FindClose(findHandle);
    MessageBox("Cannot install to the same directory as the "
               "Install program is running from.");
    return FALSE;
  }

  // Check to for sufficient free space

  if (GetFreeMB(m_mainDirectory, &freeMB)) {
    if (freeMB < 2) {
      MessageBox("Fewer than 2 MB free.  Cannot install.", "Error",
                 MB_OK | MB_ICONEXCLAMATION);
      return FALSE;
    }
    if (freeMB < DEFAULT_FREE_SPACE + 10) {
      sprintf(errorString,
              "There are only %d MB free on the chosen partition.  We "
              "recommend\n"
              "at least %d MB.  Do you want to continue with installation?",
              freeMB, DEFAULT_FREE_SPACE + 10);
      response = MessageBox(errorString, "Warning", MB_YESNO | MB_DEFBUTTON2);
      if (response != IDYES)
        return FALSE;
    }
  }

  // Check that the e-mail address given is valid

  if (m_personalName.GetLength() == 0) {
    MessageBox("You should enter an e-mail address so we can alert you to "
               "important information about VTrace.\n");
    return FALSE;
  }

  if (m_personalName.Find('@') == -1 ) {
    MessageBox("The e-mail address you entered does not have an @ in it.\n"
               "Check to make sure you entered it correctly.");
    return FALSE;
  }

  // Check to make sure max string lengths are not violated

  if (m_mainDirectory.GetLength() + 23 >= MAX_PATH) {
                            // 23 is the length of \Traces\T9999999.vtr.gz
    MessageBox("The main directory path is too long.\n"
               "Please choose a directory with a shorter name.");
    return FALSE;
  }

  // Check that main directory isn't being changed

  if (m_formerMainDirectory.GetLength() > 0 &&
      m_formerMainDirectory.CompareNoCase(m_mainDirectory)) {
    MessageBox("You can't change the main directory using the installer "
               "since VTrace is already installed to a different directory.\n"
               "You have two choices: (1) Leave the directory the same. "
               "(2) Uninstall VTrace first, then reboot, then install it "
               "with the new directory.");
    return FALSE;
  }

  return TRUE;
}
