//
// Override.cpp implements a simple Windows program that turns off VTrcLog
// logging while it is running.
//

#include <windows.h>
#include <winioctl.h>
#include <winsvc.h>
#include <string.h>
#include <stdio.h>
#include "resource.h"
#include "../Common/VTrace.h"

char szWinName[] = "MyWin"; // name of window class

////////////////////////////////////////////////////////////////////////////
//                           MESSAGE HANDLER
////////////////////////////////////////////////////////////////////////////

// WindowFunc is called by Windows NT and is passed messages from the
// message queue.

LRESULT CALLBACK WindowFunc (HWND hwnd, UINT message, WPARAM wParam,
                             LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  HBRUSH hBrush;
  TEXTMETRIC tm;
  char msg[80];

  switch(message) {
    case WM_COMMAND:
      switch(LOWORD(wParam)) {
        case IDM_EXIT:  
          PostQuitMessage(0);
          break;
      }
      break;
    case WM_PAINT:            // paint the window with our text message
      hdc = BeginPaint(hwnd, &ps);
      hBrush = (HBRUSH) GetStockObject(WHITE_BRUSH);
      SelectObject(hdc, hBrush);
      PatBlt(hdc, ps.rcPaint.left, ps.rcPaint.top,
             ps.rcPaint.right - ps.rcPaint.left,
             ps.rcPaint.bottom - ps.rcPaint.top, PATCOPY);
      GetTextMetrics(hdc, &tm);
      strcpy(msg, "As long as this program is running, VTrace is not.");
      TextOut(hdc, 10, 10, msg, strlen(msg));
      strcpy(msg, "Exit this program to restart VTrace.");
      TextOut(hdc, 10, 10 + tm.tmHeight + tm.tmExternalLeading, msg,
              strlen(msg));
      EndPaint(hwnd, &ps);
      break;
    case WM_DESTROY:        // terminate the program
      PostQuitMessage(0);
      break;
    default:
      // Let Windows NT process any messages not specified in
      // the preceding switch statement
      return DefWindowProc(hwnd, message, wParam, lParam);
  }
  return 0;
}

////////////////////////////////////////////////////////////////////////////
//                         MAIN APPLICATION WINDOW
////////////////////////////////////////////////////////////////////////////

// WinMain is the main program.  It turns off VTrcLog logging while it is
// running.

int WINAPI WinMain (HINSTANCE hThisInst, HINSTANCE hPrevInst, LPSTR lpszArgs,
                    int nWinMode)
{
  HWND hwnd;
  MSG msg;
  WNDCLASSEX wcl;
  HACCEL hAccel;
  HANDLE hVTrcLogDevice;
  DWORD reason = BECAUSE_USER_REQUESTED, length;

  // Define a window class
  wcl.cbSize = sizeof(WNDCLASSEX); 

  wcl.hInstance = hThisInst;                  // handle to this instance
  wcl.lpszClassName = szWinName;              // window class name
  wcl.lpfnWndProc = WindowFunc;               // window function
  wcl.style = 0;                              // default style
  wcl.hIcon = LoadIcon(hThisInst, MAKEINTRESOURCE(IDR_MAINFRAME));
                                              // standard icon
  wcl.hIconSm = LoadIcon(hThisInst, MAKEINTRESOURCE(IDR_MAINFRAME1));
                                              // small icon
  wcl.hCursor = LoadCursor(NULL, IDC_ARROW);  // cursor style

  // Specify name of menu resource
  wcl.lpszMenuName = MAKEINTRESOURCE(IDR_MENU1); // main menu

  // No extra information needed
  wcl.cbClsExtra = 0;
  wcl.cbWndExtra = 0;

  // Make the window white
  wcl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH); 

  // Register the window class
  if(!RegisterClassEx(&wcl)) return 0;

  // Now that a window class has been registered, a window can be created
  hwnd = CreateWindow(
    szWinName,              // name of window class
    "Override VTrace",      // title
    WS_OVERLAPPEDWINDOW,    // window style - normal
    CW_USEDEFAULT,          // X coordinate - let Windows decide
    CW_USEDEFAULT,          // Y coordinate - let Windows decide
    350,                    // width 350 pixels
    100,                    // height 100 pixels
    HWND_DESKTOP,           // no parent window
    NULL,                   // no override of class menu
    hThisInst,              // handle of this instance of the program
    NULL                    // no additional arguments
  );

  // Load accelerators
  hAccel = LoadAccelerators(hThisInst, MAKEINTRESOURCE(IDR_ACCELERATOR1));

  // Display the window
  ShowWindow(hwnd, nWinMode);
  UpdateWindow(hwnd);

  // Open a handle to the VTrcLog driver and send it a message to stop logging
  hVTrcLogDevice = CreateFile("\\\\.\\VTrcLog", GENERIC_READ | GENERIC_WRITE,
                              0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
                              NULL);
  if (hVTrcLogDevice == INVALID_HANDLE_VALUE) {
    MessageBox(NULL, "Could not connect to VTrace logger device.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
    return 0;
  }
  DeviceIoControl(hVTrcLogDevice, VTRACE_CTRL_ADD_REASONS_TO_STOP, &reason,
                  4, NULL, 0, &length, NULL);

  // Create the message loop
  while(GetMessage(&msg, NULL, 0, 0))
  {
    if(!TranslateAccelerator(hwnd, hAccel, &msg)) {
      TranslateMessage(&msg); /* translate keyboard messages */
      DispatchMessage(&msg); /* return control to Windows NT */
    }
  }

  // Now that we're about to exit, tell VTrcLog it can start logging again
  DeviceIoControl(hVTrcLogDevice, VTRACE_CTRL_REMOVE_REASONS_TO_STOP, &reason,
                  4, NULL, 0, &length, NULL);
  return msg.wParam;
}
