//
// UninstallDlg.cpp : implements the CUninstallDlg class
//

#include "stdafx.h"
#include "Uninstall.h"
#include "UninstallDlg.h"
#include <winioctl.h>
#include <winsvc.h>
#include "UninstallRoutines.h"
#include "../Common/Version.h"
#include "../Common/RegConstraints.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUninstallDlg dialog

CUninstallDlg::CUninstallDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CUninstallDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CUninstallDlg)
    // NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CUninstallDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CUninstallDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CUninstallDlg, CDialog)
  //{{AFX_MSG_MAP(CUninstallDlg)
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUninstallDlg message handlers

BOOL CUninstallDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon
  
  // TODO: Add extra initialization here
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR CUninstallDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

// CUninstallDlg::OnOK is called when the user hits OK.  It uninstalls VTrace.

void CUninstallDlg::OnOK() 
{
  SC_HANDLE schSCManager;
  char systemRoot[MAX_PATH], mainDirectory[MAX_PATH];
  BOOLEAN completeSuccess = TRUE;
  int response;
  CWnd *okButton;

  // Disable the OK button

  okButton = GetDlgItem(IDOK);
  if (okButton != NULL) okButton->EnableWindow(FALSE);

  // Request confirmation of the request

  response = MessageBox("Are you sure you want to uninstall VTrace?",
                        "Uninstall", MB_YESNO | MB_DEFBUTTON2);
  if (response != IDYES) {
    CDialog::OnOK();
    return;
  }

  // Get the name of the VTrace main directory, needed for various later things

  if (!GetMainDirectory(mainDirectory, MAX_PATH))
    return;

  // Get system root directory name, needed for various later things

  if (GetEnvironmentVariable("SystemRoot", systemRoot, 128) == 0) {
    DisplayError("Could not get SystemRoot environment variable");
    return;
  }

  // If there are at least 5 MB of trace files remaining, give the user one
  // more chance to see the error of his or her ways and upload those files
  // before uninstalling

  if (TraceMBRemaining(mainDirectory) >= 5) {
    response = MessageBox(
      "A substantial amount of trace data has not been uploaded yet.\n"
      "\n"
      "The trace data files WILL BE DELETED if you uninstall now.  And that "
      "would be sad,\n"
      "considering all you've gone through to collect this valuable data.\n"
      "\n"
      "Do you want to uninstall anyway?\n"
      "\n"
      "(If you say no, I'll cancel the uninstall for now and start "
      "the utility that lets you upload trace files.)", "Warning",
      MB_YESNO | MB_DEFBUTTON2);
    if (response == IDNO)
      LaunchUploadUtility(mainDirectory);
    if (response != IDYES) {
      CDialog::OnOK();
      return;
    }
  }

  // Stop logging

  StopLogging();

  // Delete the DLL and all references to it

  if (!DeleteDLL(systemRoot)) completeSuccess = FALSE;

  // Open service control manager in order to delete VTrace services

  schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
  if (schSCManager == NULL)
    DisplayError("Could not open service control manager to delete VTrace "
                 "services");
  else {
    // Delete drivers

    if (!DeleteDriver("VTrcInit", schSCManager, systemRoot))
      completeSuccess = FALSE;
    if (!DeleteDriver("VTrcFsrc", schSCManager, systemRoot))
      completeSuccess = FALSE;
    if (!DeleteDriver("VTrcFS", schSCManager, systemRoot))
      completeSuccess = FALSE;
#if TARGET_NT_VERSION == 5
    if (!RemoveUpperFilter("VTrcKbd", KEYBOARD_CLASS_GUID)) {
      completeSuccess = FALSE;
      // Do not remove VTrcKbd if it's still an upper filter!
    }
    else {
      if (!DeleteDriver("VTrcKbd", schSCManager, systemRoot))
        completeSuccess = FALSE;
    }
#else
    if (!DeleteDriver("VTrcKbd", schSCManager, systemRoot))
      completeSuccess = FALSE;
#endif
    if (!DeleteDriver("VTrcNet", schSCManager, systemRoot))
      completeSuccess = FALSE;
#if TARGET_NT_VERSION == 5
    if (!RemoveUpperFilter("VTrcRawD", STORAGE_VOLUME_CLASS_GUID)) {
      completeSuccess = FALSE;
      // Do not remove VTrcRawD if it's still an upper filter!
    }
    else {
      if (!DeleteDriver("VTrcRawD", schSCManager, systemRoot))
        completeSuccess = FALSE;
    }
#else
    if (!DeleteDriver("VTrcRawD", schSCManager, systemRoot))
      completeSuccess = FALSE;
#endif

    // Delete service

    if (!DeleteVTrcSrvc(schSCManager, systemRoot))
      completeSuccess = FALSE;

    // Only delete the VTrcLog driver if all things dependent on that driver
    // have been successfully deleted

    if (completeSuccess) {
      if (!DeleteDriver("VTrcLog", schSCManager, systemRoot))
        completeSuccess = FALSE;
    }
    else
      MessageBox("Not removing VTrcLog because something dependent on "
                 "it could not be deleted.", "Uninstall");

    // Close handle to service control manager

    CloseServiceHandle(schSCManager);
  }

  // Delete the main VTrace directory and all its contents

  if (completeSuccess && !DeleteMainDirectory(mainDirectory))
    completeSuccess = FALSE;

  // Remove shortcuts in Start menu

  if (completeSuccess && !DeleteStartMenuShortcuts())
    completeSuccess = FALSE;

  // Delete most of the registry info associated with VTrace

  if (completeSuccess && !DeleteRegistryInfo())
    completeSuccess = FALSE;

  // Display summary message

  if (completeSuccess)
    MessageBox("VTrace uninstalled.\n"
               "You must restart the machine for some aspects of the "
               "uninstallation to be complete.",
               "Uninstall");
  else
    MessageBox("VTrace not completely uninstalled.", "Uninstall");

  CDialog::OnOK();
}
