//
// VTraceLogDriver.cpp: implementation of the CLogDriver class.
//

#include "stdafx.h"
#include "../Common/VTrace.h"
#include "Structs.h"
#include "Upload.h"
#include "Helpers.h"
#include <winioctl.h>
#include <winsvc.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////
//                      CONSTRUCTION / DESTRUCTION
////////////////////////////////////////////////////////////////////////////

CLogDriver::CLogDriver()
{
  hDevice = (HANDLE) -1;
  accessAcquired = false;
}

CLogDriver::~CLogDriver()
{
  Close();
}

////////////////////////////////////////////////////////////////////////////
//                         SET-UP / TEAR-DOWN
////////////////////////////////////////////////////////////////////////////

// CLogDriver::Open opens a handle to the VTrcLog driver, returning a boolean
// indicating whether successful

bool CLogDriver::Open (void)
{
  if (hDevice != (HANDLE) -1) return true;

  hDevice = CreateFile("\\\\.\\VTrcLog", GENERIC_READ | GENERIC_WRITE, 0, NULL,
                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  return (hDevice != (HANDLE) -1);
}

// CLogDriver::Close closes the open handle to the VTrcLog driver, if there is
// one.  But, before it does that, it releases access to the trace files if
// it previously acquired that access and retains it.

void CLogDriver::Close (void)
{
  if (hDevice != (HANDLE) -1) {
    ReleaseAccess();
    CloseHandle(hDevice);
    hDevice = (HANDLE) -1;
  }
}

////////////////////////////////////////////////////////////////////////////
//                          ACCESS REQUESTS
////////////////////////////////////////////////////////////////////////////

// CLogDriver::AcquireAccess requests access to the trace files from the
// VTrcLog driver, and returns a boolean indicating whether access was
// acquired.  Also, if access is acquired, the field accessAcquired is
// set so that access can be releasd when the driver connection is closed.

bool CLogDriver::AcquireAccess (void)
{
  DWORD length, reasons = BECAUSE_ACCESS_REQUIRED;

  // If we haven't opened the device yet, open it.  If we can't open it, then
  // VTrace isn't running, and no one should be accessing the trace but us.
  // So, it should be OK to upload, and therefore we say we have acquired
  // access.
  
  if (!Open())
    return true;

  // Send a request for access.  If it returns success, we have acquired
  // access.

  if (DeviceIoControl(hDevice, VTRACE_CTRL_ADD_REASONS_TO_STOP, &reasons, 4,
                      NULL, 0, &length, NULL))
    accessAcquired = true;

  return accessAcquired;
}

// CLogDriver::ReleaseAccess releases access to the trace files, letting
// VTrcLog resume logging.

void CLogDriver::ReleaseAccess (void)
{
  DWORD length, reasons = BECAUSE_ACCESS_REQUIRED;

  if (accessAcquired && (hDevice != (HANDLE) -1))
    if (DeviceIoControl(hDevice, VTRACE_CTRL_REMOVE_REASONS_TO_STOP, &reasons, 4,
                        NULL, 0, &length, NULL))
      accessAcquired = false;
}

////////////////////////////////////////////////////////////////////////////
//                      SENDING DISK USAGE MESSAGE
////////////////////////////////////////////////////////////////////////////

// CLogDriver::IndicateReducedDiskUsage indicates to VTrcLog that some disk
// space has been freed up, and so various reasons why logging may have been
// stopped no longer apply.

void CLogDriver::IndicateReducedDiskUsage (void)
{
  DWORD length;
  DWORD reason = BECAUSE_DISK_FULL | BECAUSE_FLUSH_FAILED;

  // Note that the use of VTRACE_START does not really mean that VTrace will
  // start, just that these reasons for not starting will be removed

  if (hDevice != (HANDLE) -1)
    DeviceIoControl(hDevice, VTRACE_CTRL_REMOVE_REASONS_TO_STOP, &reason, 4,
                    NULL, 0, &length, NULL);
}
