//
// UploadDlg.cpp : implements the CUploadDlg class
//

#include "StdAfx.h"
#include "Structs.h"
#include "Helpers.h"
#include "Upload.h"
#include "UploadDlg.h"
#include "UploadThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUploadDlg dialog

CUploadDlg::CUploadDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CUploadDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CUploadDlg)
    // NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in
        // Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CUploadDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CUploadDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CUploadDlg, CDialog)
  //{{AFX_MSG_MAP(CUploadDlg)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_WM_KILLFOCUS()
  ON_MESSAGE(WM_CHANGE_STATUS, OnChangeStatus)
  ON_MESSAGE(WM_CHANGE_PROGRESS, OnChangeProgress)
  ON_MESSAGE(WM_UPLOAD_COMPLETE, OnUploadComplete)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUploadDlg message handlers

BOOL CUploadDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon

  uploadingThread = NULL;

  CRect progressControlRect(11, 32, 396, 47);

  // Initialize progress message

  SetDlgItemText(IDC_STATIC_PROGRESS, "Acquiring access to trace files");

  // Acquire access to trace files

  theApp.logDriver.Open();
  if (!theApp.logDriver.AcquireAccess()) {
    MessageBox("Cannot upload now because another VTrace program is "
               "accessing the trace files.  Try again later.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
    EndDialog(-1);
    return TRUE;
  }

  // Create progress meter

  progressControl.Create(WS_VISIBLE | WS_CHILD, progressControlRect, this,
                         IDC_UPLOAD_PROGRESSBAR);
  progressControl.SetRange(0, 100);
  progressControl.SetPos(0);

  // Start the asynchronous thread to upload trace files

  uploadingThread = AfxBeginThread(&UploadAllAsynchronously, m_hWnd);
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.

HCURSOR CUploadDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

// OnCancel cancels any file sending that might be going on in anticipation
// of the program ending.  It does this by terminating the file-sending thread.

void CUploadDlg::OnCancel() 
{
  if (uploadingThread != NULL) {
    TerminateThread(uploadingThread->m_hThread, -1);
    uploadingThread = NULL;
  }
  
  CDialog::OnCancel();
}

// OnChangeStatus is called when a WM_CHANGE_STATUS message is posted to the
// dialog box.  The wParam field is a pointer to a string that the status
// message should be changed to.

void CUploadDlg::OnChangeStatus (WPARAM wParam, LPARAM lParam)
{
  SetDlgItemText(IDC_STATIC_PROGRESS, (char *) wParam);
}

// OnChangeProgress is called when a WM_CHANGE_PROGRESS message is posted to
// the dialog box.  The wParam field is the value (between 0 and 100,
// inclusive) that the progress bar fullness should be set to.

void CUploadDlg::OnChangeProgress (WPARAM wParam, LPARAM lParam)
{
  progressControl.SetPos((int) wParam);
}

// OnUploadComplete is called when a WM_UPLOAD_COMPLETE message is posted to
// the dialog box.  It indicates that uploading is complete.  If uploading
// completed successfully, wParam is nonzero, so the user should be logged off
// (and the program terminated) if he or she checked the appropriate box.

void CUploadDlg::OnUploadComplete (WPARAM wParam, LPARAM lParam)
{
  uploadingThread = NULL;

  if (wParam && IsDlgButtonChecked(IDC_CHECK1))
    if (!ExitWindowsEx(EWX_LOGOFF, 0))
      DisplayError("Could not log off as requested after finishing");

  PostMessage(WM_COMMAND, IDCANCEL, 0);
}
