//
// UploadThread.cpp contains the thread routine for doing the uploading
// work asynchronously.  It also contains support functions for that
// routine.
//

#include "StdAfx.h"
#include "Upload.h"
#include "UploadDlg.h"
#include "Structs.h"
#include "Helpers.h"
#include "Compression.h"
#include "Web.h"
#include "../Common/VTrace.h"
#include <afxsock.h>
#include <windows.h>

////////////////////////////////////////////////////////////////////////////
//                       HIGH-LEVEL UPLOAD ROUTINES
////////////////////////////////////////////////////////////////////////////

// UploadAllAsynchronously takes a window handle as a parameter and calls
// the necessary routines to upload all the traces.  Its odd declaration
// format is so that it can be called by a thread creation routine.  It assumes
// that access to the trace files has already been acquired.

UINT UploadAllAsynchronously (LPVOID param)
{
  HWND hWnd = (HWND) param;
  REGISTRY_INFO regInfo;

  PostMessage(hWnd, WM_CHANGE_STATUS, (WPARAM) "Reading registry", 0);

  // Read the relevant parameters from the registry

  if (!ReadRegistry(regInfo)) {
    // On failure, post message to indicate the upload completed unsuccessfully
    PostMessage(hWnd, WM_UPLOAD_COMPLETE, 0, 0);
    return 0;
  }

  PostMessage(hWnd, WM_CHANGE_STATUS, (WPARAM) "Waiting for logging to stop",
              0);

  // Sleep to allow logging to stop

  Sleep(PAUSE_FOR_FLUSHING_MS);

  // Compress all the traces, then send them

  CompressAll(hWnd, regInfo);
  SendAll(hWnd, regInfo);

  // Post an "upload completed successfully" message

  PostMessage(hWnd, WM_UPLOAD_COMPLETE, 1, 0);
  return 0;
}
