//
// Activate.c
//
// Code for deciding whether VTrace should run this time.
//

#include "ntddk.h"
#include "Activate.h"
#include "../Common/Version.h"

//////////////////////////////////////////////////////////////////////////
//                       FUNCTION DECLARATIONS
//////////////////////////////////////////////////////////////////////////

NTSYSAPI VOID NTAPI HalDisplayString (PCHAR String);

////////////////////////////////////////////////////////////////////////////
//                                SECTIONS
////////////////////////////////////////////////////////////////////////////

// Define the sections that allow for discarding (i.e. paging) some of
// the code.

#ifdef ALLOC_PRAGMA
#pragma alloc_text (INIT, ShouldActivateVTrace)
#endif

//////////////////////////////////////////////////////////////////////////
//                         VTRACE ACTIVATION
//////////////////////////////////////////////////////////////////////////

// ShouldActivateVTrace returns a boolean indicating whether VTrace should
// be activated this time. It returns TRUE if it determines that we are
// not using the LastKnownGood configuration.

BOOLEAN ShouldActivateVTrace (void)
{
  NTSTATUS status;
  UNICODE_STRING unicodeString;
  OBJECT_ATTRIBUTES objectAttributes;
  HANDLE keyHandle;
  struct {
    ULONG   TitleIndex;
    ULONG   Type;
    ULONG   DataLength;
    WCHAR   Data[2];
  } keyValueInfo;
  ULONG keyValueInfoLength;
  ULONG current, lastKnownGood;

  // Open the registry key for selecting which configuration to start the
  // system with

  RtlInitUnicodeString(&unicodeString, L"\\Registry\\Machine\\System\\Select");
  objectAttributes.Length = sizeof(OBJECT_ATTRIBUTES);
  objectAttributes.RootDirectory = NULL;
  objectAttributes.ObjectName = &unicodeString;
  objectAttributes.SecurityDescriptor = NULL;
  objectAttributes.SecurityQualityOfService = NULL;
  objectAttributes.Attributes = OBJ_CASE_INSENSITIVE;
  status = ZwOpenKey(&keyHandle, KEY_QUERY_VALUE | KEY_SET_VALUE,
                     &objectAttributes);
  if (status == STATUS_SUCCESS) {
    // Read which configuration the current one is

    RtlInitUnicodeString(&unicodeString, L"Current");
    status = ZwQueryValueKey(keyHandle, &unicodeString,
                             KeyValuePartialInformation, &keyValueInfo,
                             sizeof(keyValueInfo), &keyValueInfoLength);
    if (status == STATUS_SUCCESS && keyValueInfo.Type == REG_DWORD) {
      current = * (ULONG *) keyValueInfo.Data;

      // Read which configuration the last-known-good one is

      RtlInitUnicodeString(&unicodeString, L"LastKnownGood");
      status = ZwQueryValueKey(keyHandle, &unicodeString,
                               KeyValuePartialInformation, &keyValueInfo,
                               sizeof(keyValueInfo), &keyValueInfoLength);
      if (status == STATUS_SUCCESS && keyValueInfo.Type == REG_DWORD) {
        lastKnownGood = * (ULONG *) keyValueInfo.Data;

        ZwClose(keyHandle);
        if (current == lastKnownGood) {
#if TARGET_NT_VERSION == 4
          HalDisplayString("VTrace deactivated because LastKnownGood "
                           "configuration is in use.\n");
#endif
          return FALSE;
        }
        else {
#if TARGET_NT_VERSION == 4
          HalDisplayString("VTrace activated.\n");
#endif
          return TRUE;
        }
      }
    }

    // Close the registry key
    ZwClose(keyHandle);
  }

  // If we failed somewhere, return FALSE.

#if TARGET_NT_VERSION == 4
  HalDisplayString("VTrace deactivated because it could not determine whether "
                   "the last known configuration was in use.\n");
#endif
  return FALSE;
}
