//
// Beat.c
//
// Code to maintain a "beat" of events that make sure that not too long
// goes by without an event (so that the 32-bit time field never overflows
// without being able to tell that it has).
//

#include "NTDDK.h"
#include "../Common/VTrace.h"
#include "../Common/VTrcDrv.h"
#include "Globals.h"
#include "Beat.h"
#include "GetState.h"
#include "LogEvent.h"

////////////////////////////////////////////////////////////////////////////
//                                SECTIONS
////////////////////////////////////////////////////////////////////////////

// Define the sections that allow for discarding (i.e. paging) some of
// the code.

#ifdef ALLOC_PRAGMA
#pragma alloc_text (INIT, InitializeBeat)
#endif

////////////////////////////////////////////////////////////////////////////
//                           BEAT FUNCTIONS
////////////////////////////////////////////////////////////////////////////

// BeatRoutine is the routine called on each beat.  It simply records
// a beat event in the log.  Its odd parameter format is to allow it to
// be a deferred procedure call.

void BeatRoutine (IN PKDPC Dpc, IN PVOID DeferredContext,
                  IN PVOID SystemArgument1, IN PVOID SystemArgument2)
{
  KIRQL oldirql;

  KeAcquireSpinLock(&sharedState->mainMutex, &oldirql);
  LogEvent((char) ENTRY_TYPE_BEAT, 1);
  KeReleaseSpinLock(&sharedState->mainMutex, oldirql);
}

// InitializeBeat initializes the beat timer so that a beat will happen
// often enough to keep time.

void InitializeBeat (void)
{
  LONG beatPeriodInMsec;
  LARGE_INTEGER beatPeriodInNeg100nsec;

  // The beat period is roughly 75% of 2^32 cycles

  beatPeriodInNeg100nsec.HighPart = 7;
  beatPeriodInNeg100nsec.LowPart = 0x80000000;
  beatPeriodInNeg100nsec.QuadPart /= globals.CPUSpeedInMHz;
  beatPeriodInNeg100nsec.QuadPart *= -1;

  beatPeriodInMsec = 3221225 / globals.CPUSpeedInMHz;

  KeInitializeDpc(&globals.beatDpc, &BeatRoutine, NULL);
  KeInitializeTimer(&globals.beatTimer);
  KeSetTimerEx(&globals.beatTimer, beatPeriodInNeg100nsec, beatPeriodInMsec,
               &globals.beatDpc);
}
