// 
// Unicode.c
//
// Code for special Unicode-related routines in the VTrcLog driver.
//

#include "NTDDK.h"

////////////////////////////////////////////////////////////////////////
//                           UNICODE STRINGS
////////////////////////////////////////////////////////////////////////

// MyInitUnicodeString allocates space for a Unicode string of at most
// maxLength characters, and fills in a Unicode string structure pointed
// to by string that refers to that space.  It returns a status value.

NTSTATUS MyInitUnicodeString (PUNICODE_STRING string, int maxLength)
{
  string->Buffer = ExAllocatePool(NonPagedPool, maxLength * 2);
                                                     // sizeof(WCHAR) is 2
  if (string->Buffer == NULL)
    return STATUS_INSUFFICIENT_RESOURCES;
  string->Length = 0;
  string->MaximumLength = maxLength * 2;

  return STATUS_SUCCESS;
}

// MyFreeUnicodeString deallocates space that was allocated for a given
// Unicode string by MyInitUnicodeString.

void MyFreeUnicodeString (PUNICODE_STRING string)
{
  if (string->Buffer != NULL) {
    ExFreePool(string->Buffer);
    string->Buffer = NULL;
    string->Length = 0;
    string->MaximumLength = 0;
  }
}

// MyClearUnicodeString sets a Unicode string to the empty string.

void MyClearUnicodeString (PUNICODE_STRING string)
{
  string->Length = 0;
}
