//
// Helpers.h contains export information for Helpers.cpp, and useful
// macros
//

///////////////////////////////////////////////////////////////////////////
//                               MACROS
///////////////////////////////////////////////////////////////////////////

// The following indicates the minimum number of milliseconds that should
// be allowed to pass between metadata snapshots:

#define METADATA_VERY_OLD_MS                   43200000 /* 12 hours */

// The following indicates the minimum amount of time the user should
// have been idle before we compress and upload trace files:

#define USER_IDLE_MS_BEFORE_COMPRESS           7200000 /* 2 hours */

// The following indicates how long we should wait to gain access to the
// trace files upon learning that someone else has access to them:

#define PAUSE_FOR_ACCESS_MS                    1200000 /* 20 minutes */

// The following indicates the maximum time that should be permitted to
// pass between instances of compressing/uploading trace files:

#define MAX_MS_BETWEEN_COMPRESSIONS            86400000 /* 1 day */

// The following indicates the maximum amount of time that should pass between
// consecutive checks to see if it's time for doing another compression once
// it is determined that the user is idle but the other factors that should
// be considered don't yet add up to it being time:

#define MAX_MS_BEFORE_TRY_ANOTHER_COMPRESS     3600000 /* 1 hour */

// The following indicates the most bytes that should be allowed to build
// up in the trace directory before we abandon our policy of not compressing/
// uploading too frequently and just immediately compress. Note that this
// doesn't override the minimum user idle time that must pass. What this means
// is that if the user has been idle long enough, but the time since the last
// compression is less than MAX_MS_BETWEEN_COMPRESSIONS, normally you wouldn't
// compress, but if there are more than this many bytes in the trace directory,
// you do:

#define MAX_BYTES_BEFORE_COMPRESS              52428800 /* 50 MB */

///////////////////////////////////////////////////////////////////////////
//                       FUNCTION PROTOTYPES
///////////////////////////////////////////////////////////////////////////

DWORD MillisecondsBetweenTimes (FILETIME beginTime, FILETIME endTime);

BOOL ReadRegistry (REGISTRY_INFO *regInfoPtr, BOOL onlyIfVersionChecks);
BOOL GetCurrentTraceFileNumber (DWORD *currentTraceFileNumberPtr);
DWORD GetMetadataAge (void);
DWORD GetUserName (HKEY hKey, char *userName, DWORD bufferSize);

DWORD GetUserIdleTime (void);
