//
// JSocket.cpp implements the JSocket class, a non-MFC, restricted version
// of CSocket with a couple of extra useful functions.  Note that the caller
// must call WSAStartup() before calling any of the routines here.
//

#include "JSocket.h"
#include <string.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////
//                      CONSTRUCTION / DESTRUCTION
////////////////////////////////////////////////////////////////////////////

JSocket::JSocket()
{
}

JSocket::~JSocket()
{
  Close();
}

////////////////////////////////////////////////////////////////////////////
//                          SET-UP / TEAR-DOWN
////////////////////////////////////////////////////////////////////////////

// Create creates a socket, and returns a boolean indicating whether
// it was successful.

bool JSocket::Create()
{
  sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  return (sock != INVALID_SOCKET);
}

// Connect opens a connection to the host of the given name at the port of the
// given number, and returns a boolean indicating whether it was successful.

bool JSocket::Connect(char *hostname, int port)
{
  LPHOSTENT hostp;
  SOCKADDR_IN server;
  int n;

  memset(&server, 0, sizeof(server));

  server.sin_family = AF_INET;
  server.sin_addr.s_addr = inet_addr(hostname);
  if (server.sin_addr.s_addr == INADDR_NONE) {
    hostp = gethostbyname(hostname);
    if (hostp == NULL) return FALSE;
    server.sin_addr.s_addr = ((LPIN_ADDR) hostp->h_addr)->s_addr;
  }

  server.sin_port = htons((u_short) port);

  n = connect(sock, (SOCKADDR *) &server, sizeof(server));
  return (n != SOCKET_ERROR);
}

// Close closes the socket.

void JSocket::Close()
{
  if (sock != INVALID_SOCKET) {
    closesocket(sock);
    sock = INVALID_SOCKET;
  }
}

////////////////////////////////////////////////////////////////////////////
//                            SEND / RECEIVE
////////////////////////////////////////////////////////////////////////////

// Send sends up to length bytes from buffer buf, and returns the number
// of bytes actually sent.

int JSocket::Send (char *buf, int length)
{
  return send(sock, buf, length, 0);
}

// SendAll sends exactly length bytes from buffer buf, and returns a boolean
// indicating if it was successful.

bool JSocket::SendAll (char *buf, int length)
{
  int n;

  while (length > 0) {
    n = Send(buf, length);
    if (n == 0 || n == SOCKET_ERROR)
      return FALSE;
    else {
      buf += n;
      length -= n;
    }
  }

  return TRUE;
}

// SendAll sends a NULL-terminated string buf, and returns a boolean
// indicating if it was successful.

bool JSocket::SendAll (char *buf)
{
  return SendAll(buf, strlen(buf));
}

// Receive receives up to length bytes into buffer buf, and returns the
// number of bytes actually received.

int JSocket::Receive (char * buf, int length)
{
  return recv(sock, buf, length, 0);
}
